@@ -9,6 +9,7 @@ cld.glht <- function(object, level = 0.05, decreasing = FALSE, ...)
 extr <- function(object) {
 
     stopifnot(object$type == "Tukey")
+    
 
     mf <- model.frame(object$model)
     if (!is.null(attr(mf, "terms"))) {
@@ -17,6 +18,14 @@ extr <- function(object) {
         tm <- try(terms(object$model))
         if (inherits(tm, "try-error")) stop("no terms component found")
     }
+    
+
+    ### <FIXME> not very nice    
+    if(any(class(object$model) == "lme")){
+      mf <- get_all_vars(tm, mf)
+    }
+    ### </FIXME>
+    
     covar <- (length(attr(tm, "term.labels")) > 1)
     y <- mf[[1L]]
     yname <- colnames(mf)[[1L]]
@@ -25,13 +34,17 @@ extr <- function(object) {
     x <- mf[[object$focus]]
     xname <- object$focus
 
+    K <- contrMat(table(x), type = "Tukey")
+    comps <- cbind(apply(K, 1, function(k) levels(x)[k == 1]),
+                   apply(K, 1, function(k) levels(x)[k == -1]))
+
     f <- if (inherits(object$model, "coxph")) predict else fitted
     lp <- f(object$model)
 
     ret <- list(y = y, yname = yname,  
                 x = x, xname = xname, 
                 weights = model.weights(mf), 
-                lp = lp, covar = covar)
+                lp = lp, covar = covar, comps = comps)
     return(ret)
 }
 
@@ -39,9 +52,24 @@ cld.summary.glht <- function(object, level = 0.05, decreasing = FALSE, ...) {
 
     ret <- extr(object)
     signif <- (object$test$pvalues < level)
-    names(signif) <- gsub("\\s", "", rownames(object$linfct))
+    # Order the levels according to its mean
+    # Tidy up: ret$y[1:length(ret$x)]], cox models concatenates a vector of live/dead
+    # I think this way is easier than to deal with gsub later and it's more general
+    lvl_order <- levels(ret$x)[order(tapply(as.numeric(ret$y)[1:length(ret$x)], ret$x, mean))]
+    # names(signif) <- gsub("\\s", "", rownames(object$linfct))
     ret$signif <- signif
-    ret$mcletters <- insert_absorb(signif, decreasing = decreasing,)
+    ret$mcletters <- insert_absorb(signif, decreasing = decreasing, 
+                                   comps = ret$comps, lvl_order = lvl_order, ...)
+                           
+   # start edit
+    
+    ret$mcletters$Letters <- ret$mcletters$Letters[levels(ret$x)]
+    ret$mcletters$monospacedLetters <- ret$mcletters$monospacedLetters[levels(ret$x)]
+    ret$mcletters$LetterMatrix <- ret$mcletters$LetterMatrix[levels(ret$x),]
+    
+   # end edit
+                           
+                          
     class(ret) <- "cld"
     ret
 }
@@ -51,9 +79,22 @@ cld.confint.glht <- function(object, decreasing = FALSE, ...) {
     ret <- extr(object)
     ### significant, if confidence interval does not contains 0
     signif <- !(object$confint[, "lwr"] < 0 & object$confint[, "upr"] > 0)
-    names(signif) <- gsub("\\s", "", rownames(object$linfct))
+    # Tidy up: ret$y[1:length(ret$x)]], cox models concatenates a vector of live/dead
+    # I think this way is easier than to deal with gsub later and it's more general
+    lvl_order <- levels(ret$x)[order(tapply(as.numeric(ret$y)[1:length(ret$x)], ret$x, mean))]
+    # names(signif) <- gsub("\\s", "", rownames(object$linfct))
     ret$signif <- signif
-    ret$mcletters <- insert_absorb(signif, decreasing = decreasing)
+    ret$mcletters <- insert_absorb(signif, decreasing = decreasing, 
+                                   comps = ret$comps, lvl_order = lvl_order, ...)
+                           
+    # start edit
+                           
+    ret$mcletters$Letters <- ret$mcletters$Letters[levels(ret$x)]
+    ret$mcletters$monospacedLetters <- ret$mcletters$monospacedLetters[levels(ret$x)]
+    ret$mcletters$LetterMatrix <- ret$mcletters$LetterMatrix[levels(ret$x),]
+                                   
+    # end edit                      
+                           
     class(ret) <- "cld" 
     ret
 }
@@ -127,18 +168,21 @@ plot.cld <- function(x, type = c("response", "lp"), ...) {
 #               the number of letters required exceeds the number of letters available
 # Decreasing ... Inverse the order of the letters 
 
-insert_absorb <- function( x, Letters=c(letters, LETTERS), separator=".", decreasing = decreasing ){
+insert_absorb <- function( x, Letters=c(letters, LETTERS), separator=".", decreasing = FALSE, 
+                           comps = NULL, lvl_order){
 
   obj_x <- deparse(substitute(x))
-  namx <- names(x)
-  namx <- gsub(" ", "", names(x))
-  if(length(namx) != length(x))
-    stop("Names required for ", obj_x)
-  split_names <- strsplit(namx, "-")
-  stopifnot( sapply(split_names, length) == 2 )
-  comps <- t(as.matrix(as.data.frame(split_names)))
+  if (is.null(comps)) {
+      namx <- names(x)
+      namx <- gsub(" ", "", names(x))
+      if(length(namx) != length(x))
+          stop("Names required for ", obj_x)
+      split_names <- strsplit(namx, "-")
+      stopifnot( sapply(split_names, length) == 2 )
+      comps <- t(as.matrix(as.data.frame(split_names)))
+  } 
   rownames(comps) <- names(x)
-  lvls <- unique(as.vector(comps))
+  lvls <- lvl_order
   n <- length(lvls)
   lmat <- array(TRUE, dim=c(n,1), dimnames=list(lvls, NULL) )
 
